/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/12/29
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for 
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

/*
 *@Note
 *task1 and task2 alternate printing
 */

#include "Periphs.h"
#include "debug.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "NEO.h"

/* Global define */
#define TASK1_TASK_PRIO     5
#define TASK1_STK_SIZE      256
#define TASK2_TASK_PRIO     5
#define TASK2_STK_SIZE      256

/* Global Variable */
TaskHandle_t      Strip1Task_Handler;
SemaphoreHandle_t Strip1Semph;
TaskHandle_t Task2Task_Handler;
TaskHandle_t AnimTask_Handler;

extern uint32_t Blank[ ];
extern uint32_t Strip[ ];

strip Neo1;

/*********************************************************************
 * @fn      GPIO_Toggle_INIT
 * @brief   Initializes GPIOA.0/1
 * @return  none
 */
void GPIO_Toggle_INIT(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure={0};

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA,ENABLE);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void CH_Delay_Us( uint32_t d ) {

    for( uint64_t cd=0; cd<5*d;cd++) {
    	asm("nop"); asm("nop"); asm("nop"); asm("nop");
    	asm("nop"); asm("nop"); asm("nop"); asm("nop");
    }

}

/*********************************************************************
 * @fn      Anim task
 * @brief   Animation task program.
 * @param  *pvParameters - Parameters point of task1
 * @return  none
 */

void anim_task(void *pvParameters)
{
	printf("Anim Task Setup\n");

    while(1)
    {
        if( xSemaphoreTake( Strip1Semph, ( TickType_t ) 20 ) == pdTRUE )
        	 { /* printf("Anim: 81 Take OK\n"); */ }
        else { printf("Anim: 82 Take FAILED!!!\n"); }

        /* printf("movSeg_R\n"); */  moveSeg_R( &Neo1, 0, false, 10 ); //  vTaskDelay( 200 );

        if( xSemaphoreGive( Strip1Semph ) == pdTRUE )
        	 { /* printf("Anim: 87 Give OK\n"); */ }
        else { printf("Anim: 88 Give FAILED\n"); }
        vTaskDelay( 40 );
    }
}

void Blink_task(void *pvParameters)
{

    while(1)
    {
//        printf("X1 = %d\r\n", X1++ );
        GPIO_SetBits(GPIOA, GPIO_Pin_0);
        vTaskDelay(208);
        GPIO_ResetBits(GPIOA, GPIO_Pin_0);
        vTaskDelay(279);
    }
}

/*********************************************************************
 * @fn      NEO display task
 * @brief   task2 program.
 * @param  *pvParameters - Parameters point of task2
 * @return  none
 */
void Strip1_task(void *pvParameters)
{
	printf("Strip1 Task Setup\n\n");

    Create_pStrip( &Neo1, 15, BLU, NULL );

//    Strip1Semph = xSemaphoreCreateBinary();

    if( Strip1Semph == NULL )
    	{ printf("Semaphore creation failed - Memory?\n"); }

    if( xSemaphoreGive( Strip1Semph ) == pdTRUE )
    	 { /* printf("Strip1: 124 Give OK\n"); */ }
    else { printf("Strip1: 125 Give FAIL!!\n"); }


    while(1)
    {
        if( xSemaphoreTake( Strip1Semph, ( TickType_t ) 4000 ) == pdTRUE )
        	{ /* printf("Strip1: 131 Take OK\n"); **/ }
        else { printf("Strip1: 132 Take FAIL!!\n"); }

//    	moveSeg_R( &Neo1, 0, false, 10 );
//    	printf("Y1 = %d\r\n", Y1++ );
//    	SPI_Send( 0x4664 );
        /* printf("Compress5\n"); */ /**/ Compress5( &Neo1 ); /**/
//        GPIO_ResetBits(GPIOA, GPIO_Pin_1);
//        vTaskDelay( 1 );
// 	SPI_Send( 0x4664 );
//        GPIO_SetBits(GPIOA, GPIO_Pin_1);
//        vTaskDelay( 1 );

        if( xSemaphoreGive( Strip1Semph ) == pdFALSE )
        	 { printf("Strip1: 145 GIVE FAIL!!\n"); }
        else { /* printf("Strip1: 146 GIVE OK\n"); */ }

        vTaskDelay( 50 );

    }
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
    SystemCoreClockUpdate();
    Delay_Init();
    USART_Printf_Init(115200);
    printf("\nCanHobby Periphs RTOS Clk=%d MHz\n",SystemCoreClock / 1000000 );
    printf( "ChipID:%08x\n", DBGMCU_GetCHIPID() );
    ChipID();
    printf("FreeRTOS Kernel Version:%s\n",tskKERNEL_VERSION_NUMBER);

    GPIO_Toggle_INIT();

//    SPI_1Lines_HalfDuplex_Init();
    Init_SPIs(  SPI1_msk ); // | SPI2_msk | SPI3_msk );
//    while(1) SPI_Send( 0x4664 );

//    Create_pStrip( &Neo1, 12, -1, Strip );

    Clear( 15 );
    Delay_Ms( 500 );

    Neo1.segs[0][START] = 3;
    Neo1.segs[0][END] = 8;
    Neo1.segs[0][SIZE] = 5;
    Neo1.seg_d[0] = Blank;
    Neo1.segs[0][YY] = 3;

//    Compress5( &Neo1 );
/***  while(1) {
	 CH_Delay_Us( 120 );
	Delay_Ms( 1200 );
	printf(".\n");
 }
 ****/
    /* create two task */
    Strip1Semph = xSemaphoreCreateBinary();

    xTaskCreate((TaskFunction_t )Blink_task,
                    (const char*    )"Blink",
                    (uint16_t       )TASK1_STK_SIZE,
                    (void*          )NULL,
                    (UBaseType_t    )TASK1_TASK_PRIO,
                    (TaskHandle_t*  )&Strip1Task_Handler);
/***/
    xTaskCreate((TaskFunction_t )anim_task,
                    (const char*    )"Anim",
                    (uint16_t       )TASK1_STK_SIZE,
                    (void*          )NULL,
                    (UBaseType_t    )TASK2_TASK_PRIO,
                    (TaskHandle_t*  )&AnimTask_Handler );

    xTaskCreate((TaskFunction_t )Strip1_task,
                        (const char*    )"NEO display",
                        (uint16_t       )TASK2_STK_SIZE,
                        (void*          )NULL,
                        (UBaseType_t    )TASK2_TASK_PRIO,
                        (TaskHandle_t*  )&Task2Task_Handler);

/***/
    vTaskStartScheduler();
//
 /* ***/
    while(1);
    {
//    	SPI_Send( 0x4664 );
        printf("shouldn't get at here!!\n");
    }
}

